@@ -7,7 +7,7 @@ OC.L10N.register(
     "No file provided for import" : "未提供导入文件",
     "Unsupported file type for import" : "不支持导入的文件类型",
     "A Bookmark manager for Nextcloud" : "适用于 Nextcloud 的书签管理",
-    "This app allows you to store and organize your favorite places on the web in one spot, while allowing you to sync them with your various devices and browsers.\n\nCheck out the third-party clients listed here: https://github.com/nextcloud/bookmarks#third-party-clients" : "本应用允许您一站式地存储和整理您喜爱的网站，同时允许您将他们同步到您不同的设备和浏览器中。\n\n在这里查看第三方客户端：https://github.com/nextcloud/bookmarks#third-party-clients",
+    "This app provides you with a web interface for collecting and organizing bookmarks to the sites on the web that are precious to you. You can browse and filter your bookmarks via tags and folders and by using the built-in search feature. Furthermore, in order to access your bookmarks anywhere, it also allows you to synchronize third-party clients via a built-in REST API -- in your browsers and on your phone.\n\nCheck out the third-party clients listed here: https://github.com/nextcloud/bookmarks#third-party-clients" : "此应用为您提供一个Web界面，用于收集和组织对您有价值的网站书签。您可以通过标签和文件夹来浏览和过滤您的书签并使用内置的搜索功能。另外，为使您能从任何地点访问您的书签，它允许您通过内建的REST API与第三方客户端同步——在您的浏览器和您的手机上。\n\n请在这里查看第三方客户端：https://github.com/nextcloud/bookmarks#third-party-clients",
     "Details" : "详情",
     "Rename" : "重命名",
     "Move" : "移动",
@@ -49,6 +49,8 @@ OC.L10N.register(
     "Screeenly" : "Screeenly",
     "Screeenly API URL" : "Screenly API 地址",
     "Screeenly API key" : "Screenly API 密钥",
+    "Privacy" : "隐私",
+    "Performance" : "性能",
     "Failed to load settings" : "无法加载设置",
     "Failed to save settings" : "无法保存设置",
     "Add a bookmark" : "添加书签",
