@@ -19,19 +19,19 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  *
  */
-import Vue from 'vue';
-import Tooltip from 'nextcloud-vue/dist/Directives/Tooltip';
-import App from './components/ViewAdmin';
-import store from './store/';
-import AppGlobal from './mixins/AppGlobal';
+import Vue from 'vue'
+import Tooltip from 'nextcloud-vue/dist/Directives/Tooltip'
+import App from './components/ViewAdmin'
+import store from './store/'
+import AppGlobal from './mixins/AppGlobal'
 
-Vue.mixin(AppGlobal);
-Vue.directive('tooltip', Tooltip);
+Vue.mixin(AppGlobal)
+Vue.directive('tooltip', Tooltip)
 
 const BookmarksApp = (global['Bookmarks'] = new Vue({
 	el: '#bookmarks',
 	store,
 	render: h => h(App)
-}));
+}))
 
-export default BookmarksApp;
+export default BookmarksApp
